/*
 * System Interface Library for games
 * Copyright (c) 2011 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/test/sysdep/darwin/time.c: Tests for Darwin (OSX/iOS) timekeeping
 * functions.
 */

#include "src/base.h"
#define IN_SYSDEP  // So we get the real functions instead of the diversions.
#include "src/sysdep.h"
#include "src/sysdep/darwin/time.h"
#include "src/test/base.h"

#include <mach/mach.h>
#include <mach/mach_time.h>

/*************************************************************************/
/****************************** Test runner ******************************/
/*************************************************************************/

DEFINE_GENERIC_TEST_RUNNER(test_darwin_time)

/*************************************************************************/
/***************************** Test routines *****************************/
/*************************************************************************/

TEST(test_time_unit)
{
    mach_timebase_info_data_t timebase_info;
    mach_timebase_info(&timebase_info);
    uint64_t ticks_per_sec = ((1000000000 * (uint64_t)timebase_info.denom
                               + (uint64_t)(timebase_info.numer / 2))
                              / (uint64_t)timebase_info.numer);
    int time_shift = 0;
    while (ticks_per_sec >= UINT64_C(1) << 31) {
        time_shift++;
        ticks_per_sec >>= 1;
    }

    CHECK_TRUE(sys_time_init() == (int64_t)ticks_per_sec);

    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_delay)
{
    mach_timebase_info_data_t timebase_info;
    mach_timebase_info(&timebase_info);
    uint64_t ticks_per_sec = ((1000000000 * (uint64_t)timebase_info.denom
                               + (uint64_t)(timebase_info.numer / 2))
                              / (uint64_t)timebase_info.numer);
    const int ticks_per_csec = ticks_per_sec / 100;
    const int time_unit = sys_time_init();
    uint64_t start, end;

    start = mach_absolute_time();
    end = mach_absolute_time();
    CHECK_INTRANGE(end - start, 0, ticks_per_csec/2);

    start = mach_absolute_time();
    sys_time_delay(time_unit/100);
    end = mach_absolute_time();
    CHECK_INTRANGE(end - start, ticks_per_csec, ticks_per_csec*2);

    start = mach_absolute_time();
    sys_time_delay(0);
    end = mach_absolute_time();
    CHECK_INTRANGE(end - start, 0, ticks_per_csec/2);

    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_epoch)
{
    const int time_unit = sys_time_init();
    const uint64_t now = sys_time_now();
    CHECK_DOUBLEEQUAL(darwin_time_epoch(), (double)now / (double)time_unit);

    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_time_from_timestamp)
{
    sys_time_init();
    const uint64_t now = sys_time_now();
    CHECK_DOUBLEEQUAL(darwin_time_from_timestamp(now), 0.0);

    return 1;
}

/*************************************************************************/
/*************************************************************************/

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
